/******************************************************************************
* Module	:	Tiles --- Lay out tiles to solve a string ordering problem.
*
* Document	:	\section{Tiles}
*			:	
*			:	The problem to solve is how to lay out a number of tiles,
*			:	in a fixed size vector space and with no unfilled spots
*			:	in the result.
*			:	
*			:	There are a fixed number of tiles, such that each tile
*			:	can be represented by a vector whose contents are:
*			:	
*			:	tile = 'N' + N * Space + 'N'
*			:	
*			:	where Space is the ASCII space character, and 'N' is
*			:	the ASCII character that represents the number N.
*			:	
*			:	Tiles are laid on top of each other such that an 'N'
*			:	can only be placed within a space character.
*			:	
*			:	The vector into which tiles are placed satisfies:
*			:	
*			:	Vector = Space * (2 * N + 2)  iff zero is used, or
*			:	Vector = Space * (2 * N)      if  zero is not used.
*			:	
*			:	This program was put together to solve a problem given
*			:	to me by Mark Linroth: Mentor, Life Long Friend and
*			:	a Man of Many Parts.
*			:	
*			:	This program is a brute force search program.  This
*			:	version is much faster than the MGA for small tile
*			:	sizes, but it ends up being much slower for larger
*			:	tile sizes.
*			:	
* EndDocum	:	
******************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>

void
bruteForceSearch(
	char	*vector,
	int		vectorSize,
	int		tileMaximum,
	int		tileSize,
	int		baseOne )
{
	register	int		i;
	register	int		j;
	auto		int		maxOffset;
	auto		char	c;

	/*	Is tile size greater than 9?	*/
	if (tileSize < 10)
		c = '0' + tileSize;
	else
		c = 'a' + tileSize - 10;

	/*	Calculate the maximum offset.	*/
	maxOffset = vectorSize - tileSize - 2;
	j         = tileSize + 1;

	/*	For all the possible tile locations, do:	*/
	for (i = 0; i <= maxOffset; i++)
	{
		/*	Is the location viable?	*/
		if (vector[ i ] == ' ' && vector[i + j] == ' ')
		{
			/*	Yes.  Set the vector values.	*/
			vector[ i     ] = c;
			vector[ i + j ] = c;

			/*	Is this the last tile to place?	*/
			if (baseOne == tileSize)
			{
				/*	Yes!  Found a tiling!	*/
				printf(
					"%s\n",
					vector );
			}
			else
			{
				/*	No.  Recurse.	*/
				bruteForceSearch(
					vector,
					vectorSize,
					tileMaximum,
					tileSize - 1,
					baseOne );
			}

			/*	Unset the vector values.	*/
			vector[ i     ] = ' ';
			vector[ i + j ] = ' ';
		}
	}
}

int
main(
	int		argc,
	char	**argv )
{
	auto	int		totalTiles  = 0;
	auto	int		baseOne     = 0;
	auto	char	*vector	    = 0;
	auto	char	*score	    = 0;
	auto	int		vectorSize  = 0;
	auto	int		tileMaximum = 0;

	/*	Do we have enough command line parameters?	*/
	if (argc < 3)
	{
		/*	No.  Print error and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - syntax is\n"
			"tiles <maximum tile size> <base one>\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	Get command line parameters.	*/
	tileMaximum =
		atoi(
			argv[ 1 ] );
	baseOne     =
		atoi(
			argv[ 2 ] );
	baseOne     =
		( baseOne )
			?	1
			:	0 ;

	/*	Calculate the vector size.	*/
	vectorSize = tileMaximum * 2 + 2 * (1 - baseOne);

	/*	Can we allocate a vector?	*/
	if ( (vector =
		calloc(
			1,
			vectorSize + 1 )) == 0 )
	{
		/*	No.  Print error and exit.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	Set vector to be all spaces.	*/
	memset(
		vector,
		' ',
		vectorSize );
	vector[ vectorSize ] = 0;

	/*	Do brute force search.	*/
	bruteForceSearch(
		vector,
		vectorSize,
		tileMaximum,
		tileMaximum,
		baseOne );
}
